// SWIG input file of the 'fem' module

%feature("autodoc","1");

%module(docstring=
"'femi' module: FEM
(c) ULiège - A&M",
threads="1"
) femi
%{

#include "fem.h"
#include "femDirichlet.h"
#include "femNeumann.h"
#include "femProblem.h"
#include "femSolver.h"
#include "femPost.h"
#include "femMedium.h"
#include "femExtractor.h"

#include <string>
// #include <sstream>

%}

%include "std_string.i"
%include "std_vector.i"
%include "std_map.i"

// Instantiate templates
namespace std {
   %template(StringDoubleMap)  std::map<std::string, double>;
   %template(StringVector)  std::vector<std::string>;
   %template(DoubleVector)  std::vector<double>;
}

// --------- EXCEPTIONS ---------------

%include "exception.i"

// from: http://swig.10945.n7.nabble.com/Trapping-Swig-DirectorException-td6013.html
// le code suivant permet de voir la call stack dans les appels C++ => python

%{ 
   static void handle_exception(void) { 
     try { 
       throw; 
     } catch (std::exception &e) { 
        std::stringstream txt; 
        txt << e.what(); // << ' ' << typeid(e).name();
        PyErr_SetString(PyExc_RuntimeError, e.what()); 
     } 
     catch(...) 
     {
        PyErr_SetString(PyExc_RuntimeError, "Unknown C++ Runtime Error");
     } 
   } 
%} 

%exception { 
   try { 
     $action 
   } catch (...) { 
     // Note that if a director method failed, the Python error indicator 
     // already contains full details of the exception, and it will be 
     // reraised when we go to SWIG_fail; so no need to convert the C++ 
     // exception back to a Python one 
     if (!PyErr_Occurred()) { 
       handle_exception(); 
     } 
     SWIG_fail; 
   } 
} 

// ----------- FEM CLASSES ---------------
%include "fem.h"
%include "femDirichlet.h"
%include "femNeumann.h"
%include "femSolver.h"
%include "femProblem.h"
%include "femPost.h"
%include "femMedium.h"
