#ifndef FEMPROBLEM_H
#define FEMPROBLEM_H

#include "fem.h"
#include <map>

namespace fem
{

/// contains:
/// - entities: a list of entities
/// - groups: a list of physical groups

class FEM_API Problem
{
public:
#ifndef SWIG
    std::vector<Entity *> entities;
    std::map<std::pair<int, int>, Entity *> entities_by_dimtag;

    std::vector<Group *> groups;
    std::map<std::pair<int, int>, Group *> groups_by_dimtag;
    std::map<std::string, Group *> groups_by_name;

    std::vector<std::size_t> nodeTags;
    std::map<size_t, size_t> nodemap; ///< node tag => index in "nodeTags"

    std::vector<Medium *> media;
    std::vector<Dirichlet *> dirichlets;
    std::vector<Neumann *> neumanns;

    std::string quadrature;
#endif
public:
    Problem();
    ~Problem();

    void sync();
    void load(std::string const &filename,
              std::map<std::string, double> const &pars, 
              int order=1);

#ifndef SWIG
        friend FEM_API std::ostream &operator<<(std::ostream &out, Problem const &obj);
#endif
};

} // namespace fem

#endif // FEMPROBLEM_H
